package com.mistdev.notificationscheduler

import android.content.Context
import androidx.test.ext.junit.runners.AndroidJUnit4
import androidx.test.platform.app.InstrumentationRegistry
import org.junit.Assert.assertEquals
import org.junit.Test
import org.junit.runner.RunWith
import java.util.*

@RunWith(AndroidJUnit4::class)
class ShouldShowNotificationTests {

    private val appContext: Context = InstrumentationRegistry.getInstrumentation().context

    @Test
    fun testShouldShowNotification_shouldNot_beforeDate() {
        val details = NotificationDetails(0, "Conta", "Pagar conta", 10, 5, 0)
        val calendar = Calendar.getInstance().apply {
            set(2019, Calendar.JANUARY, 4)
        }

        val scheduler = NotificationScheduler(appContext)
        val shouldShow = scheduler.shouldShowNotification(details, calendar)
        assertEquals(shouldShow, false)
    }

    @Test
    fun testShouldShowNotification_shouldNot_afterDate() {
        val details = NotificationDetails(0, "Conta", "Pagar conta", 10, 5, 0)
        val calendar = Calendar.getInstance().apply {
            set(2019, Calendar.JANUARY, 11)
        }

        val scheduler = NotificationScheduler(appContext)
        val shouldShow = scheduler.shouldShowNotification(details, calendar)
        assertEquals(shouldShow, false)
    }

    @Test
    fun testShouldShowNotification_shouldNot_dateEnd_januaryToFebruary() {
        val details = NotificationDetails(0, "Conta", "Pagar conta", 29, 5, 0)
        val calendar = Calendar.getInstance().apply {
            set(2019, Calendar.JANUARY, 30)
        }

        val scheduler = NotificationScheduler(appContext)
        val shouldShow = scheduler.shouldShowNotification(details, calendar)
        assertEquals(shouldShow, false)
    }

    @Test
    fun testShouldShowNotification_should_atDateStart() {
        val details = NotificationDetails(0, "Conta", "Pagar conta", 10, 5, 0)
        val calendar = Calendar.getInstance().apply {
            set(2019, Calendar.JANUARY, 5)
        }

        val scheduler = NotificationScheduler(appContext)
        val shouldShow = scheduler.shouldShowNotification(details, calendar)
        assertEquals(shouldShow, true)
    }

    @Test
    fun testShouldShowNotification_should_atDateEnd() {
        val details = NotificationDetails(0, "Conta", "Pagar conta", 10, 5, 0)
        val calendar = Calendar.getInstance().apply {
            set(2019, Calendar.JANUARY, 10)
        }

        val scheduler = NotificationScheduler(appContext)
        val shouldShow = scheduler.shouldShowNotification(details, calendar)
        assertEquals(shouldShow, true)
    }

    @Test
    fun testShouldShowNotification_should_february() {
        val details = NotificationDetails(0, "Conta", "Pagar conta", 30, 5, 0)
        val calendar = Calendar.getInstance().apply {
            set(2019, Calendar.FEBRUARY, 27)
        }

        val scheduler = NotificationScheduler(appContext)
        val shouldShow = scheduler.shouldShowNotification(details, calendar)
        assertEquals(shouldShow, true)
    }

    @Test
    fun testShouldShowNotification_should_previousMonth() {
        val details = NotificationDetails(0, "Conta", "Pagar conta", 1, 5, 0)
        val calendar = Calendar.getInstance().apply {
            set(2019, Calendar.FEBRUARY, 24)
        }

        val scheduler = NotificationScheduler(appContext)
        val shouldShow = scheduler.shouldShowNotification(details, calendar)
        assertEquals(shouldShow, true)
    }
}