package com.mistdev.notificationscheduler

import android.app.Notification
import android.app.NotificationManager
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import androidx.core.app.NotificationCompat
import android.app.NotificationChannel
import android.os.Build

class NotificationBroadcastReceiver : BroadcastReceiver() {
    companion object {
        const val EXTRA_NOTIFICATION_ID = "EXTRA_NOTIFICATION_ID"
        const val EXTRA_NOTIFICATION_TITLE = "EXTRA_NOTIFICATION_TITLE"
        const val EXTRA_NOTIFICATION_MESSAGE = "EXTRA_NOTIFICATION_MESSAGE"
        const val EXTRA_NOTIFICATION_DUE_DAY = "EXTRA_NOTIFICATION_DUE_DAY"
        const val EXTRA_NOTIFICATION_DAYS_TO_NOTIFY = "EXTRA_NOTIFICATION_DAYS_TO_NOTIFY"
        const val EXTRA_NOTIFICATION_ICON = "EXTRA_NOTIFICATION_ICON"

        const val NOTIFICATION_CHANNEL_ID = "notification_scheduler_channel"
        const val NOTIFICATION_CHANNEL_NAME = "Contas"
    }

    override fun onReceive(context: Context, intent: Intent) {
        val scheduler = NotificationScheduler(context)
        val details = buildNotificationDetailsFromExtras(intent)

        if(scheduler.shouldShowNotification(details)) {
            val notificationManager = context.getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                val channel = NotificationChannel(NOTIFICATION_CHANNEL_ID, NOTIFICATION_CHANNEL_NAME, NotificationManager.IMPORTANCE_HIGH)
                notificationManager.createNotificationChannel(channel)
            }

            notificationManager.notify(
                    details.id,
                    buildNotification(context, details)
            )

        } else {
            scheduler.apply {
                cancelScheduledNotification(details)
                scheduleNotification(details)
            }
        }
    }

    private fun buildNotificationDetailsFromExtras(intent: Intent) : NotificationDetails {
        val id = intent.getIntExtra(EXTRA_NOTIFICATION_ID, 0)
        val title = intent.getStringExtra(EXTRA_NOTIFICATION_TITLE)
        val message = intent.getStringExtra(EXTRA_NOTIFICATION_MESSAGE)
        val dueDay = intent.getIntExtra(EXTRA_NOTIFICATION_DUE_DAY, 0)
        val daysToNotify = intent.getIntExtra(EXTRA_NOTIFICATION_DAYS_TO_NOTIFY, 0)
        val icon = intent.getIntExtra(EXTRA_NOTIFICATION_ICON, 0)
        return NotificationDetails(id, title, message, dueDay, daysToNotify, icon)
    }

    private fun buildNotification(context: Context, details: NotificationDetails) : Notification {
        return NotificationCompat.Builder(context, NOTIFICATION_CHANNEL_ID)
                .setContentTitle(details.title)
                .setContentText(details.message)
                .setSmallIcon(details.smallIconId)
                .setAutoCancel(true)
                .build()
    }
}