package com.mistdev.notificationscheduler

import android.app.AlarmManager
import android.app.PendingIntent
import android.content.Context
import android.content.Intent
import java.util.*

class NotificationScheduler(context: Context) : INotificationScheduler {
    companion object {
        const val DEFAULT_ALARM_HOUR = 10
    }

    private val appContext = context.applicationContext

    override fun scheduleNotification(notificationDetails: NotificationDetails, now: Calendar) {
        val alarmManager = appContext.getSystemService(Context.ALARM_SERVICE) as AlarmManager

        val alarmIntent = buildAlarmIntent(appContext, notificationDetails)
        val startDate = NotificationDateBuilder.buildNextNotificationStartDate(notificationDetails, now, DEFAULT_ALARM_HOUR)

        alarmManager.setInexactRepeating(
                AlarmManager.RTC,
                startDate.timeInMillis,
                AlarmManager.INTERVAL_DAY,
                alarmIntent
        )
    }

    override fun cancelScheduledNotification(notificationDetails: NotificationDetails) {
        val alarmIntent = buildAlarmIntent(appContext, notificationDetails)

        val alarmManager = appContext.getSystemService(Context.ALARM_SERVICE) as AlarmManager
        alarmManager.cancel(alarmIntent)
    }

    override fun shouldShowNotification(details: NotificationDetails, now: Calendar): Boolean {
        val end = NotificationDateBuilder.buildEndDate(details, now)
        val start = NotificationDateBuilder.buildStartDate(details, end)

        return (now.before(end) && now.after(start)) ||
                now.timeInMillis == start.timeInMillis ||
                now.timeInMillis == end.timeInMillis
    }

    private fun buildAlarmIntent(context: Context, details: NotificationDetails) : PendingIntent {
        val notificationIntent = Intent(context, NotificationBroadcastReceiver::class.java)
        notificationIntent.putExtra(NotificationBroadcastReceiver.EXTRA_NOTIFICATION_ID, details.id)
        notificationIntent.putExtra(NotificationBroadcastReceiver.EXTRA_NOTIFICATION_TITLE, details.title)
        notificationIntent.putExtra(NotificationBroadcastReceiver.EXTRA_NOTIFICATION_MESSAGE, details.message)
        notificationIntent.putExtra(NotificationBroadcastReceiver.EXTRA_NOTIFICATION_DUE_DAY, details.dueDay)
        notificationIntent.putExtra(NotificationBroadcastReceiver.EXTRA_NOTIFICATION_DAYS_TO_NOTIFY, details.daysToNotify)
        notificationIntent.putExtra(NotificationBroadcastReceiver.EXTRA_NOTIFICATION_ICON, details.smallIconId)
        return PendingIntent.getBroadcast(context, details.id, notificationIntent, PendingIntent.FLAG_CANCEL_CURRENT)
    }

}