package com.mistdev.notificationscheduler

import android.content.ComponentName
import android.content.Context
import android.content.pm.PackageManager
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar

class NotificationSchedulerPlugin(private val appContext: Context): MethodCallHandler {
    companion object {
        private const val CHANNEL_NAME = "com.mistdev/notification_scheduler"

        const val METHOD_GET_PLATFORM_VERSION = "getPlatformVersion"
        const val METHOD_SCHEDULE_NOTIFICATION = "scheduleNotification"
        const val METHOD_CANCEL_NOTIFICATION = "cancelNotification"

        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), CHANNEL_NAME)
            channel.setMethodCallHandler(NotificationSchedulerPlugin(registrar.activity().applicationContext))
        }
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when(call.method) {
            METHOD_GET_PLATFORM_VERSION -> result.success("Android ${android.os.Build.VERSION.RELEASE}")
            METHOD_SCHEDULE_NOTIFICATION -> scheduleNotification(call, result)
            METHOD_CANCEL_NOTIFICATION -> cancelNotification(call, result)
            else -> result.notImplemented()
        }
    }

    private fun scheduleNotification(call: MethodCall, result: Result) {
        try {
            val notificationDetails = NotificationDetails.build(appContext, call)

            NotificationScheduler(appContext).scheduleNotification(notificationDetails)

            enableDontKillApp()

        } catch (e: Exception) {
            result.error(e.message, null, null)
            return
        }

        result.success(true)
    }

    private fun cancelNotification(call: MethodCall, result: Result) {
        try {
            val notificationDetails = NotificationDetails.build(appContext, call)

            NotificationScheduler(appContext).cancelScheduledNotification(notificationDetails)

            disableDontKillApp()

        } catch (e: Exception) {
            result.error(e.message, null, null)
            return
        }

        result.success(true)
    }

    /**
     * Keep the alarm set even after rebooting the phone
     */
    private fun enableDontKillApp() {
        val receiver = ComponentName(appContext, NotificationBroadcastReceiver::class.java)
        appContext.packageManager.setComponentEnabledSetting(
                receiver,
                PackageManager.COMPONENT_ENABLED_STATE_ENABLED,
                PackageManager.DONT_KILL_APP
        )
    }

    /**
     * Do NOT keep the alarm set after rebooting the phone
     */
    private fun disableDontKillApp() {
        val receiver = ComponentName(appContext, NotificationBroadcastReceiver::class.java)
        appContext.packageManager.setComponentEnabledSetting(
                receiver,
                PackageManager.COMPONENT_ENABLED_STATE_DISABLED,
                PackageManager.DONT_KILL_APP
        )
    }
}
