import 'package:flutter/material.dart';
import 'dart:async';

import 'package:flutter/services.dart';
import 'package:notification_scheduler/notification_scheduler.dart';
import 'package:notification_scheduler/notification_details.dart';

void main() => runApp(MyApp());

class MyApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  String _platformVersion = 'Unknown';
  bool _isScheduled = false;

  @override
  void initState() {
    super.initState();
    initPlatformState();
  }

  // Platform messages are asynchronous, so we initialize in an async method.
  Future<void> initPlatformState() async {
    String platformVersion;
    bool isScheduled = false;

    // Platform messages may fail, so we use a try/catch PlatformException.
    try {
      platformVersion = await NotificationScheduler.platformVersion;
    } on PlatformException {
      platformVersion = 'Failed to get platform version.';
    }

    try {

    	final details = NotificationDetails(
					1,
					"\$ Aluguel Spot",
					"Vencimento: dia 25",
					25,
					5,
					null
			);

      isScheduled = await NotificationScheduler.scheduleNotification(details);

    } on PlatformException {
      print("# FAILED: scheduleNotification()");
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    setState(() {
      _platformVersion = platformVersion;
      _isScheduled = isScheduled;
    });
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          title: const Text('Plugin example app'),
        ),
        body: Center(
          child: Text('Running on: $_platformVersion\nScheduled: $_isScheduled'),
        ),
      ),
    );
  }
}
